"""Added fields for ATAK plugin versions

Revision ID: 8d6f811f4fd7
Revises: ab719ee9b322
Create Date: 2025-11-06 15:26:31.710563

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '8d6f811f4fd7'
down_revision = 'ab719ee9b322'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    try:
        with op.batch_alter_table('packages', schema=None) as batch_op:
            batch_op.add_column(sa.Column('atak_version', sa.String(length=255), nullable=True))
    except BaseException as e:
        print(e)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('packages', schema=None) as batch_op:
        batch_op.drop_column('atak_version')

    # ### end Alembic commands ###
