# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: atak.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\natak.proto\x12\nmeshtastic\"\xe6\x01\n\tTAKPacket\x12\x15\n\ris_compressed\x18\x01 \x01(\x08\x12$\n\x07\x63ontact\x18\x02 \x01(\x0b\x32\x13.meshtastic.Contact\x12 \n\x05group\x18\x03 \x01(\x0b\x32\x11.meshtastic.Group\x12\"\n\x06status\x18\x04 \x01(\x0b\x32\x12.meshtastic.Status\x12\x1e\n\x03pli\x18\x05 \x01(\x0b\x32\x0f.meshtastic.PLIH\x00\x12#\n\x04\x63hat\x18\x06 \x01(\x0b\x32\x13.meshtastic.GeoChatH\x00\x42\x11\n\x0fpayload_variant\"2\n\x07GeoChat\x12\x0f\n\x07message\x18\x01 \x01(\x0c\x12\x0f\n\x02to\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x42\x05\n\x03_to\"M\n\x05Group\x12$\n\x04role\x18\x01 \x01(\x0e\x32\x16.meshtastic.MemberRole\x12\x1e\n\x04team\x18\x02 \x01(\x0e\x32\x10.meshtastic.Team\"\x19\n\x06Status\x12\x0f\n\x07\x62\x61ttery\x18\x01 \x01(\r\"4\n\x07\x43ontact\x12\x10\n\x08\x63\x61llsign\x18\x01 \x01(\x0c\x12\x17\n\x0f\x64\x65vice_callsign\x18\x02 \x01(\x0c\"_\n\x03PLI\x12\x12\n\nlatitude_i\x18\x01 \x01(\x0f\x12\x13\n\x0blongitude_i\x18\x02 \x01(\x0f\x12\x10\n\x08\x61ltitude\x18\x03 \x01(\x05\x12\r\n\x05speed\x18\x04 \x01(\r\x12\x0e\n\x06\x63ourse\x18\x05 \x01(\r*\xc0\x01\n\x04Team\x12\x14\n\x10Unspecifed_Color\x10\x00\x12\t\n\x05White\x10\x01\x12\n\n\x06Yellow\x10\x02\x12\n\n\x06Orange\x10\x03\x12\x0b\n\x07Magenta\x10\x04\x12\x07\n\x03Red\x10\x05\x12\n\n\x06Maroon\x10\x06\x12\n\n\x06Purple\x10\x07\x12\r\n\tDark_Blue\x10\x08\x12\x08\n\x04\x42lue\x10\t\x12\x08\n\x04\x43yan\x10\n\x12\x08\n\x04Teal\x10\x0b\x12\t\n\x05Green\x10\x0c\x12\x0e\n\nDark_Green\x10\r\x12\t\n\x05\x42rown\x10\x0e*\x7f\n\nMemberRole\x12\x0e\n\nUnspecifed\x10\x00\x12\x0e\n\nTeamMember\x10\x01\x12\x0c\n\x08TeamLead\x10\x02\x12\x06\n\x02HQ\x10\x03\x12\n\n\x06Sniper\x10\x04\x12\t\n\x05Medic\x10\x05\x12\x13\n\x0f\x46orwardObserver\x10\x06\x12\x07\n\x03RTO\x10\x07\x12\x06\n\x02K9\x10\x08\x42_\n\x13\x63om.geeksville.meshB\nATAKProtosZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_TEAM = DESCRIPTOR.enum_types_by_name['Team']
Team = enum_type_wrapper.EnumTypeWrapper(_TEAM)
_MEMBERROLE = DESCRIPTOR.enum_types_by_name['MemberRole']
MemberRole = enum_type_wrapper.EnumTypeWrapper(_MEMBERROLE)
Unspecifed_Color = 0
White = 1
Yellow = 2
Orange = 3
Magenta = 4
Red = 5
Maroon = 6
Purple = 7
Dark_Blue = 8
Blue = 9
Cyan = 10
Teal = 11
Green = 12
Dark_Green = 13
Brown = 14
Unspecifed = 0
TeamMember = 1
TeamLead = 2
HQ = 3
Sniper = 4
Medic = 5
ForwardObserver = 6
RTO = 7
K9 = 8


_TAKPACKET = DESCRIPTOR.message_types_by_name['TAKPacket']
_GEOCHAT = DESCRIPTOR.message_types_by_name['GeoChat']
_GROUP = DESCRIPTOR.message_types_by_name['Group']
_STATUS = DESCRIPTOR.message_types_by_name['Status']
_CONTACT = DESCRIPTOR.message_types_by_name['Contact']
_PLI = DESCRIPTOR.message_types_by_name['PLI']
TAKPacket = _reflection.GeneratedProtocolMessageType('TAKPacket', (_message.Message,), {
  'DESCRIPTOR' : _TAKPACKET,
  '__module__' : 'atak_pb2'
  # @@protoc_insertion_point(class_scope:meshtastic.TAKPacket)
  })
_sym_db.RegisterMessage(TAKPacket)

GeoChat = _reflection.GeneratedProtocolMessageType('GeoChat', (_message.Message,), {
  'DESCRIPTOR' : _GEOCHAT,
  '__module__' : 'atak_pb2'
  # @@protoc_insertion_point(class_scope:meshtastic.GeoChat)
  })
_sym_db.RegisterMessage(GeoChat)

Group = _reflection.GeneratedProtocolMessageType('Group', (_message.Message,), {
  'DESCRIPTOR' : _GROUP,
  '__module__' : 'atak_pb2'
  # @@protoc_insertion_point(class_scope:meshtastic.Group)
  })
_sym_db.RegisterMessage(Group)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
  'DESCRIPTOR' : _STATUS,
  '__module__' : 'atak_pb2'
  # @@protoc_insertion_point(class_scope:meshtastic.Status)
  })
_sym_db.RegisterMessage(Status)

Contact = _reflection.GeneratedProtocolMessageType('Contact', (_message.Message,), {
  'DESCRIPTOR' : _CONTACT,
  '__module__' : 'atak_pb2'
  # @@protoc_insertion_point(class_scope:meshtastic.Contact)
  })
_sym_db.RegisterMessage(Contact)

PLI = _reflection.GeneratedProtocolMessageType('PLI', (_message.Message,), {
  'DESCRIPTOR' : _PLI,
  '__module__' : 'atak_pb2'
  # @@protoc_insertion_point(class_scope:meshtastic.PLI)
  })
_sym_db.RegisterMessage(PLI)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.geeksville.meshB\nATAKProtosZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _TEAM._serialized_start=569
  _TEAM._serialized_end=761
  _MEMBERROLE._serialized_start=763
  _MEMBERROLE._serialized_end=890
  _TAKPACKET._serialized_start=27
  _TAKPACKET._serialized_end=257
  _GEOCHAT._serialized_start=259
  _GEOCHAT._serialized_end=309
  _GROUP._serialized_start=311
  _GROUP._serialized_end=388
  _STATUS._serialized_start=390
  _STATUS._serialized_end=415
  _CONTACT._serialized_start=417
  _CONTACT._serialized_end=469
  _PLI._serialized_start=471
  _PLI._serialized_end=566
# @@protoc_insertion_point(module_scope)
