"""Added CasEvac fields

Revision ID: 3a7ea7ead17b
Revises: d68964dab66b
Create Date: 2025-10-03 04:02:44.032178

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '3a7ea7ead17b'
down_revision = 'd68964dab66b'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('casevac', schema=None) as batch_op:
        batch_op.add_column(sa.Column('convenience', sa.Integer(), nullable=True))
        batch_op.add_column(sa.Column('urgent_surgical', sa.String(length=255), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('casevac', schema=None) as batch_op:
        batch_op.drop_column('urgent_surgical')
        batch_op.drop_column('convenience')

    # ### end Alembic commands ###
